#include "stdafx.h"
#include "LTree.h"
#include "LVector.h"
#include "TransMatrix.h"
#include <math.h>
#include <time.h>

#pragma warning(disable : 4244)

#define PI 3.1415926535
#define PI2 2*PI

#define d2r(deg) ((deg*PI2)/360)
#define r2d(rad) ((rad*360)/PI2)

LTree::LTree(HDC hDc, int maxXY, int maxYZ, int maxXZ, double scaleFac, int cntL, int cntLev)
{
	m_hDc = hDc;
	m_hPen = ::CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	m_hBrush = ::CreateSolidBrush(DRAW_COLOR);
	::SetPolyFillMode(m_hDc, WINDING);
	::SelectObject(m_hDc, m_hPen);
	::SelectObject(m_hDc, m_hBrush);

	maxXYAngle = maxXY;	maxXZAngle = maxXZ;	maxYZAngle = maxYZ;
	scaleFactor = scaleFac;
	cntLimbs = cntL;
	cntLevels = cntLev;
}

LTree::~LTree()
{
	::DeleteObject(m_hPen);
	::DeleteObject(m_hBrush);
}

void LTree::DrawTransVector(LVector transPt, LVector vec)
{
	POINT* poly = new POINT[4];
	LVector resPoint(transPt.Add(vec));
	int width;

	width = ((-transPt.e3)/30) + 3;
	width /= 2;


	poly[0].x = transPt.e1 - width; 
	poly[0].y = transPt.e2; 

	poly[1].x = transPt.e1 + width; 
	poly[1].y = transPt.e2;


	width = (-resPoint.e3)/30 + 3;
	width /= 2;

	poly[2].x = resPoint.e1 + width;
	poly[2].y = resPoint.e2;

	poly[3].x = resPoint.e1 - width;
	poly[3].y = resPoint.e2;

	::Polygon(m_hDc, poly, 4);

	delete[] poly;
}

void LTree::GenerateTree(LVector startPt, LVector vec, int level)
{
	double alpha;	//rotation in XY plane
	double beta;	//rotation in XZ plane
	double gama;	//rotation in YZ plane
	double scale;	//scaling factor

	int sign;
	LVector resVec;
	int lmbCnt;

	ATLTRACE("start: %f %f %f\n", startPt.e1, startPt.e2, startPt.e3);
	ATLTRACE("vec  : %f %f %f\n", vec.e1, vec.e2, vec.e3);
	ATLTRACE("level - %d\n", level);
	if (level > cntLevels) return;

	srand((unsigned)time(NULL));

	lmbCnt = cntLimbs;
	if (cntLimbs == 0) lmbCnt = rand() % 20;
	
	for (int i=0; i<lmbCnt; i++)
	{
		//generate vector rotation angles and scaling factor
		alpha = d2r((rand() % maxXYAngle));

		sign = rand() > ((double) RAND_MAX/2.0) ? 1 : -1;
		beta = d2r((rand() % maxXZAngle));

		sign = rand() > ((double) RAND_MAX/2.0) ? 1 : -1;
		gama = d2r((rand() % maxYZAngle));

		scale = scaleFactor;
		if (scaleFactor == 0) scale = 0.8*((double) rand() / (double) (RAND_MAX));

		//compute resulting vector
		resVec = vec;

		//compute vector direction...
		resVec = GetXYRotationMatrix(alpha).Multiply(resVec);
		resVec = GetXZRotationMatrix(beta).Multiply(resVec);
		resVec = GetYZRotationMatrix(gama).Multiply(resVec);

		//scale the vector
		resVec = resVec.Multiply(scale);

		//draw the vector
		DrawTransVector(startPt, resVec);

		//call me again
		GenerateTree(startPt.Add(resVec), resVec, level + 1);
	}
}
TransMatrix LTree::GetXYRotationMatrix(double angle)
{
	TransMatrix res
	(	
		cos(angle),	sin(angle),0,
		-sin(angle),cos(angle),	0,
		0,			0,			1
	);

	return(res);
}

TransMatrix LTree::GetXZRotationMatrix(double angle)
{
	TransMatrix res
	(	
		cos(angle),	0,	sin(angle),
		0,			1,	0,
		-sin(angle),0,	cos(angle)
	);

	return(res);
}

TransMatrix LTree::GetYZRotationMatrix(double angle)
{
	TransMatrix res
	(	
		1,	0,			0,
		0,	cos(angle),	sin(angle),
		0,	-sin(angle),cos(angle)
	);

	return(res);
}
